"use strict";

exports.__esModule = true;
exports.resolveStyles = exports.chakra = void 0;

var _vue = require("vue");

var _styledSystem = require("@chakra-ui/styled-system");

var _utils = require("@chakra-ui/utils");

var _css2 = require("@emotion/css");

var _system = require("./system.attrs");

var _system2 = require("./system.utils");

var _useChakra = require("./composables/use-chakra");

var _vueUtils = require("@chakra-ui/vue-utils");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var chakraProps = {
  as: [String, Object],
  __css: Object,
  sx: Object,
  css: Object,
  noOfLines: _vueUtils.SNAO,
  baseStyle: Object,
  isTruncated: Boolean,
  layerStyle: String,
  textStyle: String,
  apply: String,
  label: String
};

/**
 * Chakra factory serves as an object of chakra enabled HTML elements,
 * and also a function that can be used to enable custom component receive chakra's style props.
 * @param tag Tag or Component
 * @param options resolver options
 * 
 * How does it work?
 *
 * 1. Components returned from the chakra factory can be styled after consuming them
 *    @example
 *    ```js
 *    const Form = chakra('form') // returns a VNode you can use in the template directly
 *    ```
 * 
 * 2. Chakra components can directly be styled upon creation using the options object of type `StyleResolverProps`
 *    This resolves style object for component styles defined in the theme.
 * 
 *    Styling components using the chakra factory function can be done using the following keys from the theme:
 *    - `baseStyle`
 *    - `layerStyle`
 *    - `textStyle`
 * 
 *    @example
 *    ```js
 *    const MyCustomButton = chakra('button', {
 *     baseStyle: {
         bg: 'papayawhip,
         color: 'red.500,
         px: 4,
         py: 3
       }
 *    })
 *    ```
 *    ```html
 *    <my-custom-button>Hello Papaya Button</my-custom-button>
 *    ```
 * 
 *    See more about the style resolution in the `resolveStyles` function.
 * 
 * 3. Chakra components created and styled using the `chakra` factory can be overriden in the template by applying
 *    style properties directly
 * 
 *    @example
 *    ```html
 *    <my-custom-button bg="blue.400">
 *      Papaya button goes blue
 *    </my-custom-button>
 *    ```
 */
// @ts-expect-error
var chakra = function chakra(tag, options) {
  if (options === void 0) {
    options = {};
  }

  return (0, _vue.defineComponent)({
    name: "chakra-factory-" + String(tag),
    inheritAttrs: false,
    props: chakraProps,
    setup: function setup(props, _ref) {
      var slots = _ref.slots,
          attrs = _ref.attrs;
      return function () {
        var inheritedClass = attrs["class"],
            __label = attrs.__label,
            rest = _objectWithoutPropertiesLoose(attrs, ["class", "__label"]);

        var _options = options,
            layerStyle = _options.layerStyle,
            baseStyle = _options.baseStyle,
            textStyle = _options.textStyle,
            noOfLines = _options.noOfLines,
            isTruncated = _options.isTruncated,
            __css = _options.__css,
            css = _options.css,
            sx = _options.sx,
            apply = _options.apply,
            label = _options.label,
            otherStyles = _objectWithoutPropertiesLoose(_options, ["layerStyle", "baseStyle", "textStyle", "noOfLines", "isTruncated", "__css", "css", "sx", "apply", "label"]); // Separate component style attributes from raw HTML attributes


        var _extractStyleAttrs = (0, _system.extractStyleAttrs)(_extends({}, otherStyles, rest)),
            styles = _extractStyleAttrs.styles,
            elementAttributes = _extractStyleAttrs.attrs;

        var theme = (0, _useChakra.useTheme)();
        var layerStyle$ = (0, _vue.computed)(function () {
          var _options2;

          return props.layerStyle || ((_options2 = options) == null ? void 0 : _options2.layerStyle);
        });
        var textStyle$ = (0, _vue.computed)(function () {
          var _options3;

          return props.textStyle || ((_options3 = options) == null ? void 0 : _options3.textStyle);
        });
        var baseStyle$ = (0, _vue.computed)(function () {
          var _options4;

          return props.baseStyle || ((_options4 = options) == null ? void 0 : _options4.baseStyle);
        });
        var noOfLines$ = (0, _vue.computed)(function () {
          var _options5;

          return props.noOfLines || ((_options5 = options) == null ? void 0 : _options5.noOfLines);
        });
        var isTruncated$ = (0, _vue.computed)(function () {
          var _options6;

          return props.isTruncated || ((_options6 = options) == null ? void 0 : _options6.isTruncated);
        });

        var __css$ = (0, _vue.computed)(function () {
          var _options7;

          return props.__css || ((_options7 = options) == null ? void 0 : _options7.__css);
        });

        var css$ = (0, _vue.computed)(function () {
          var _options8;

          return props.css || ((_options8 = options) == null ? void 0 : _options8.css);
        });
        var sx$ = (0, _vue.computed)(function () {
          var _options9;

          return props.sx || ((_options9 = options) == null ? void 0 : _options9.sx);
        });
        var apply$ = (0, _vue.computed)(function () {
          var _options10;

          return props.apply || ((_options10 = options) == null ? void 0 : _options10.apply);
        });
        var resolvedComponentStyles = resolveStyles(_extends({
          __css: __css$.value,
          baseStyle: baseStyle$.value,
          apply: apply$.value,
          layerStyle: layerStyle$.value,
          noOfLines: noOfLines$.value,
          isTruncated: isTruncated$.value,
          textStyle: textStyle$.value,
          sx: sx$.value,
          css: css$.value
        }, styles, {
          theme: theme
        }));
        var componentLabel = label || __label;
        var className = (0, _css2.css)(resolvedComponentStyles);

        var _componentName = componentLabel ? "chakra-" + componentLabel : '';

        var componentOrTag = props.as || tag; // if tag is not a dom element like as="div" and an object (vue component as an object) like v-bind:as="RouterLink"

        if (!(0, _utils.isObject)(componentOrTag) && !_system2.domElements.includes(componentOrTag)) {
          // it's a string like as="router-link"
          componentOrTag = (0, _vue.resolveComponent)(componentOrTag);
        }

        return (0, _vue.h)(componentOrTag || props.as, _extends({
          "class": (0, _utils.cx)(inheritedClass, _componentName, className)
        }, elementAttributes), slots);
      };
    }
  });
};

exports.chakra = chakra;

var resolveStyles = function resolveStyles(resolvers) {
  if (resolvers === void 0) {
    resolvers = {};
  }

  var _resolvers = resolvers,
      layerStyle = _resolvers.layerStyle,
      baseStyle = _resolvers.baseStyle,
      textStyle = _resolvers.textStyle,
      noOfLines = _resolvers.noOfLines,
      isTruncated = _resolvers.isTruncated,
      __css = _resolvers.__css,
      cssProp = _resolvers.css,
      sx = _resolvers.sx,
      apply = _resolvers.apply,
      theme = _resolvers.theme,
      otherStyles = _objectWithoutPropertiesLoose(_resolvers, ["layerStyle", "baseStyle", "textStyle", "noOfLines", "isTruncated", "__css", "css", "sx", "apply", "theme"]);

  var _layerStyle = (0, _utils.memoizedGet)(theme, "layerStyles." + layerStyle, {});

  var _textStyle = (0, _utils.memoizedGet)(theme, "textStyles." + textStyle, {});

  var truncateStyle = {};

  if (noOfLines != null) {
    truncateStyle = {
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      display: '-webkit-box',
      WebkitBoxOrient: 'vertical',
      WebkitLineClamp: noOfLines
    };
  } else if (isTruncated) {
    truncateStyle = {
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      whiteSpace: 'nowrap'
    };
  }

  var finalStyles = (0, _styledSystem.css)(Object.assign({}, __css, baseStyle, {
    apply: apply
  }, _layerStyle, _textStyle, truncateStyle, otherStyles, sx))(theme);
  var cssObject = Object.assign(finalStyles, (0, _utils.isFunction)(cssProp) ? cssProp(theme) : cssProp);
  return cssObject;
};

exports.resolveStyles = resolveStyles;

_system2.domElements.forEach(function (tag) {
  chakra[tag] = chakra(tag, {});
});
//# sourceMappingURL=chakra.js.map