function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import { computed, defineComponent, h, resolveComponent } from 'vue';
import { css } from '@chakra-ui/styled-system';
import { cx, isFunction, isObject, memoizedGet as get } from '@chakra-ui/utils';
import { css as _css } from '@emotion/css';
import { extractStyleAttrs } from './system.attrs';
import { domElements } from './system.utils';
import { useTheme } from './composables/use-chakra';
import { SNAO } from '@chakra-ui/vue-utils';
var chakraProps = {
  as: [String, Object],
  __css: Object,
  sx: Object,
  css: Object,
  noOfLines: SNAO,
  baseStyle: Object,
  isTruncated: Boolean,
  layerStyle: String,
  textStyle: String,
  apply: String,
  label: String
};

/**
 * Chakra factory serves as an object of chakra enabled HTML elements,
 * and also a function that can be used to enable custom component receive chakra's style props.
 * @param tag Tag or Component
 * @param options resolver options
 * 
 * How does it work?
 *
 * 1. Components returned from the chakra factory can be styled after consuming them
 *    @example
 *    ```js
 *    const Form = chakra('form') // returns a VNode you can use in the template directly
 *    ```
 * 
 * 2. Chakra components can directly be styled upon creation using the options object of type `StyleResolverProps`
 *    This resolves style object for component styles defined in the theme.
 * 
 *    Styling components using the chakra factory function can be done using the following keys from the theme:
 *    - `baseStyle`
 *    - `layerStyle`
 *    - `textStyle`
 * 
 *    @example
 *    ```js
 *    const MyCustomButton = chakra('button', {
 *     baseStyle: {
         bg: 'papayawhip,
         color: 'red.500,
         px: 4,
         py: 3
       }
 *    })
 *    ```
 *    ```html
 *    <my-custom-button>Hello Papaya Button</my-custom-button>
 *    ```
 * 
 *    See more about the style resolution in the `resolveStyles` function.
 * 
 * 3. Chakra components created and styled using the `chakra` factory can be overriden in the template by applying
 *    style properties directly
 * 
 *    @example
 *    ```html
 *    <my-custom-button bg="blue.400">
 *      Papaya button goes blue
 *    </my-custom-button>
 *    ```
 */
// @ts-expect-error
export var chakra = function chakra(tag, options) {
  if (options === void 0) {
    options = {};
  }

  return defineComponent({
    name: "chakra-factory-" + String(tag),
    inheritAttrs: false,
    props: chakraProps,

    setup(props, _ref) {
      var {
        slots,
        attrs
      } = _ref;
      return () => {
        var {
          class: inheritedClass,
          __label
        } = attrs,
            rest = _objectWithoutPropertiesLoose(attrs, ["class", "__label"]);

        var {
          label
        } = options,
            otherStyles = _objectWithoutPropertiesLoose(options, ["layerStyle", "baseStyle", "textStyle", "noOfLines", "isTruncated", "__css", "css", "sx", "apply", "label"]); // Separate component style attributes from raw HTML attributes


        var {
          styles,
          attrs: elementAttributes
        } = extractStyleAttrs(_extends({}, otherStyles, rest));
        var theme = useTheme();
        var layerStyle$ = computed(() => {
          var _options;

          return props.layerStyle || ((_options = options) == null ? void 0 : _options.layerStyle);
        });
        var textStyle$ = computed(() => {
          var _options2;

          return props.textStyle || ((_options2 = options) == null ? void 0 : _options2.textStyle);
        });
        var baseStyle$ = computed(() => {
          var _options3;

          return props.baseStyle || ((_options3 = options) == null ? void 0 : _options3.baseStyle);
        });
        var noOfLines$ = computed(() => {
          var _options4;

          return props.noOfLines || ((_options4 = options) == null ? void 0 : _options4.noOfLines);
        });
        var isTruncated$ = computed(() => {
          var _options5;

          return props.isTruncated || ((_options5 = options) == null ? void 0 : _options5.isTruncated);
        });

        var __css$ = computed(() => {
          var _options6;

          return props.__css || ((_options6 = options) == null ? void 0 : _options6.__css);
        });

        var css$ = computed(() => {
          var _options7;

          return props.css || ((_options7 = options) == null ? void 0 : _options7.css);
        });
        var sx$ = computed(() => {
          var _options8;

          return props.sx || ((_options8 = options) == null ? void 0 : _options8.sx);
        });
        var apply$ = computed(() => {
          var _options9;

          return props.apply || ((_options9 = options) == null ? void 0 : _options9.apply);
        });
        var resolvedComponentStyles = resolveStyles(_extends({
          __css: __css$.value,
          baseStyle: baseStyle$.value,
          apply: apply$.value,
          layerStyle: layerStyle$.value,
          noOfLines: noOfLines$.value,
          isTruncated: isTruncated$.value,
          textStyle: textStyle$.value,
          sx: sx$.value,
          css: css$.value
        }, styles, {
          theme
        }));
        var componentLabel = label || __label;

        var className = _css(resolvedComponentStyles);

        var _componentName = componentLabel ? "chakra-" + componentLabel : '';

        var componentOrTag = props.as || tag; // if tag is not a dom element like as="div" and an object (vue component as an object) like v-bind:as="RouterLink"

        if (!isObject(componentOrTag) && !domElements.includes(componentOrTag)) {
          // it's a string like as="router-link"
          componentOrTag = resolveComponent(componentOrTag);
        }

        return h(componentOrTag || props.as, _extends({
          class: cx(inheritedClass, _componentName, className)
        }, elementAttributes), slots);
      };
    }

  });
};
export var resolveStyles = function resolveStyles(resolvers) {
  if (resolvers === void 0) {
    resolvers = {};
  }

  var {
    layerStyle,
    baseStyle,
    textStyle,
    noOfLines,
    isTruncated,
    __css,
    css: cssProp,
    sx,
    apply,
    theme
  } = resolvers,
      otherStyles = _objectWithoutPropertiesLoose(resolvers, ["layerStyle", "baseStyle", "textStyle", "noOfLines", "isTruncated", "__css", "css", "sx", "apply", "theme"]);

  var _layerStyle = get(theme, "layerStyles." + layerStyle, {});

  var _textStyle = get(theme, "textStyles." + textStyle, {});

  var truncateStyle = {};

  if (noOfLines != null) {
    truncateStyle = {
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      display: '-webkit-box',
      WebkitBoxOrient: 'vertical',
      WebkitLineClamp: noOfLines
    };
  } else if (isTruncated) {
    truncateStyle = {
      overflow: 'hidden',
      textOverflow: 'ellipsis',
      whiteSpace: 'nowrap'
    };
  }

  var finalStyles = css(Object.assign({}, __css, baseStyle, {
    apply: apply
  }, _layerStyle, _textStyle, truncateStyle, otherStyles, sx))(theme);
  var cssObject = Object.assign(finalStyles, isFunction(cssProp) ? cssProp(theme) : cssProp);
  return cssObject;
};
domElements.forEach(tag => {
  chakra[tag] = chakra(tag, {});
});
//# sourceMappingURL=chakra.js.map